@@ -50,7 +50,9 @@ MatrixFunction::MatrixFunction() : MathFunction("matrix", 3) {
 int MatrixFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
 	size_t rows = (size_t) vargs[0].number().uintValue();
 	size_t columns = (size_t) vargs[1].number().uintValue();
+	if((rows > 1000 || columns > 1000) && vargs[0].number() * vargs[1].number() > Number(1, 1, 6)) return 0;
 	mstruct.clearMatrix(); mstruct.resizeMatrix(rows, columns, m_zero);
+	if(mstruct.rows() < rows || mstruct.columns() < columns) return 0;
 	size_t r = 1, c = 1;
 	for(size_t i = 0; i < vargs[2].size(); i++) {
 		if(r > rows || c > columns) {
@@ -85,6 +87,7 @@ int RankFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 		if(!mvector.rankVector(vargs[1].number().getBoolean())) return 0;
 		mstruct.clearMatrix();
 		mstruct.resizeMatrix(rows, cols, m_zero);
+		if(mstruct.rows() < rows || mstruct.columns() < cols) return 0;
 		for(size_t i = 0; i < mvector.size(); i++) {
 			mstruct[i / cols][i % cols] = mvector[i];
 		}
@@ -109,17 +112,73 @@ MergeVectorsFunction::MergeVectorsFunction() : MathFunction("mergevectors", 1, -
 int MergeVectorsFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
 	mstruct.clearVector();
 	for(size_t i = 0; i < vargs.size(); i++) {
-		if(CALCULATOR->aborted()) return 0;
 		if(vargs[i].isVector()) {
 			for(size_t i2 = 0; i2 < vargs[i].size(); i2++) {
+				if(CALCULATOR->aborted()) return 0;
 				mstruct.addChild(vargs[i][i2]);
 			}
 		} else {
+			if(CALCULATOR->aborted()) return 0;
 			mstruct.addChild(vargs[i]);
 		}
 	}
 	return 1;
 }
+VertCatFunction::VertCatFunction() : MathFunction("vertcat", 1, -1) {
+	setArgumentDefinition(1, new MatrixArgument(""));
+	setArgumentDefinition(2, new MatrixArgument(""));
+}
+int VertCatFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
+	mstruct = vargs[0];
+	for(size_t i = 1; i < vargs.size(); i++) {
+		if(vargs[i].columns() != mstruct.columns()) {
+			CALCULATOR->error(true, _("Vertical concatenation requires equal number of columns."), NULL);
+			if(i > 1) {
+				mstruct.transform(this);
+				for(; i < vargs.size(); i++) mstruct.addChild(vargs[i]);
+				return 1;
+			}
+			return 0;
+		}
+		for(size_t i2 = 0; i2 < vargs[i].size(); i2++) {
+			if(CALCULATOR->aborted()) return 0;
+			mstruct.addChild(vargs[i][i2]);
+		}
+	}
+	return 1;
+}
+HorzCatFunction::HorzCatFunction() : MathFunction("horzcat", 1, -1) {
+	setArgumentDefinition(1, new MatrixArgument(""));
+	setArgumentDefinition(2, new MatrixArgument(""));
+}
+bool HorzCatFunction::representsScalar(const MathStructure &vargs) const {return false;}
+bool HorzCatFunction::representsNonMatrix(const MathStructure &vargs) const {
+	for(size_t i = 0; i < vargs.size(); i++) {
+		if(!vargs[i].representsNonMatrix()) return false;
+	}
+	return true;
+}
+int HorzCatFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
+	mstruct = vargs[0];
+	for(size_t i = 1; i < vargs.size(); i++) {
+		if(vargs[i].rows() != mstruct.rows()) {
+			CALCULATOR->error(true, _("Horizontal concatenation requires equal number of rows."), NULL);
+			if(i > 1) {
+				mstruct.transform(this);
+				for(; i < vargs.size(); i++) mstruct.addChild(vargs[i]);
+				return 1;
+			}
+			return 0;
+		}
+		for(size_t i2 = 0; i2 < vargs[i].size(); i2++) {
+			for(size_t i3 = 0; i3 < vargs[i][i2].size(); i3++) {
+				if(CALCULATOR->aborted()) return 0;
+				mstruct[i2].addChild(vargs[i][i2][i3]);
+			}
+		}
+	}
+	return 1;
+}
 MatrixToVectorFunction::MatrixToVectorFunction() : MathFunction("matrix2vector", 1) {
 	setArgumentDefinition(1, new MatrixArgument());
 }
@@ -154,30 +213,53 @@ int ColumnFunction::calculate(MathStructure &mstruct, const MathStructure &vargs
 	return 1;
 }
 RowsFunction::RowsFunction() : MathFunction("rows", 1) {
-	setArgumentDefinition(1, new VectorArgument(""));
+	setArgumentDefinition(1, new MatrixArgument(""));
 }
 int RowsFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	if(!vargs[0].isMatrix()) mstruct = m_one;
-	else mstruct = (int) vargs[0].rows();
+	mstruct.set((long int) vargs[0].rows(), 1L, 0L);
 	return 1;
 }
 ColumnsFunction::ColumnsFunction() : MathFunction("columns", 1) {
-	setArgumentDefinition(1, new VectorArgument(""));
+	setArgumentDefinition(1, new MatrixArgument(""));
 }
 int ColumnsFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	if(!vargs[0].isMatrix()) mstruct = (int) vargs[0].countChildren();
-	else mstruct = (int) vargs[0].columns();
+	mstruct.set((long int) vargs[0].columns(), 1L, 0L);
 	return 1;
 }
 ElementsFunction::ElementsFunction() : MathFunction("elements", 1) {
-	setArgumentDefinition(1, new VectorArgument(""));
+	setArgumentDefinition(1, new MatrixArgument("", false));
 }
-int ElementsFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	if(vargs[0].isMatrix()) {
-		mstruct = (int) (vargs[0].rows() * vargs[0].columns());
-	} else {
-		mstruct = (int) vargs[0].countChildren();
+int ElementsFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
+	mstruct = vargs[0];
+	mstruct.eval(eo);
+	if(!mstruct.isMatrix()) {
+		if(mstruct.isVector() && (mstruct.size() == 0 || mstruct[0].representsScalar())) {
+			mstruct.set((long int) mstruct.size(), 1L, 0L);
+			return 1;
+		} else if(mstruct.representsScalar()) {
+			mstruct = m_one;
+			return 1;
+		} else if(eo.approximation == APPROXIMATION_EXACT || eo.approximation == APPROXIMATION_EXACT_VARIABLES) {
+			EvaluationOptions eo2 = eo;
+			eo2.approximation = APPROXIMATION_APPROXIMATE;
+			MathStructure m2(vargs[0]);
+			CALCULATOR->beginTemporaryStopMessages();
+			m2.eval(eo2);
+			if(CALCULATOR->endTemporaryStopMessages()) return -1;
+			if(m2.isMatrix()) {
+				mstruct.set((long int) (m2.rows() * m2.columns()), 1L, 0L);
+				return 1;
+			} else if(m2.isVector() && (m2.size() == 0 || m2[0].representsScalar())) {
+				mstruct.set((long int) m2.size(), 1L, 0L);
+				return 1;
+			} else if(m2.representsScalar()) {
+				mstruct = m_one;
+				return 1;
+			}
+		}
+		return -1;
 	}
+	mstruct.set((long int) (mstruct.rows() * mstruct.columns()), 1L, 0L);
 	return 1;
 }
 ElementFunction::ElementFunction() : MathFunction("element", 2, 3) {
@@ -186,49 +268,107 @@ ElementFunction::ElementFunction() : MathFunction("element", 2, 3) {
 	setArgumentDefinition(3, new IntegerArgument("", ARGUMENT_MIN_MAX_NONE, true, true, INTEGER_TYPE_SIZE));
 	setDefaultValue(3, "0");
 }
+bool ElementFunction::representsScalar(const MathStructure &vargs) const {
+	if(vargs.size() >= 2 && vargs[0].isVector() && vargs[1].isInteger() && vargs[1].number().isPositive()) {
+		if(vargs.size() == 2 || vargs[2].isZero()) {
+			if(vargs[1].number() <= vargs[0].size()) return vargs[0][vargs[1].number().uintValue() - 1].representsScalar();
+		} else if(vargs[0].isMatrix() && vargs[1].number() <= vargs[0].size() && vargs[2].isInteger() && vargs[2].number().isPositive() && vargs[2].number() <= vargs[0][0].size()) {
+			return vargs[0][vargs[1].number().uintValue() - 1][vargs[2].number().uintValue() - 1].representsScalar();
+		}
+	}
+	return false;
+}
+bool ElementFunction::representsNonMatrix(const MathStructure &vargs) const {
+	if(vargs.size() >= 2 && vargs[0].isVector() && vargs[1].isInteger() && vargs[1].number().isPositive()) {
+		if(vargs.size() == 2 || vargs[2].isZero()) {
+			if(vargs[1].number() <= vargs[0].size()) return vargs[0][vargs[1].number().uintValue() - 1].representsNonMatrix();
+		} else if(vargs[0].isMatrix() && vargs[1].number() <= vargs[0].size() && vargs[2].isInteger() && vargs[2].number().isPositive() && vargs[2].number() <= vargs[0][0].size()) {
+			return vargs[0][vargs[1].number().uintValue() - 1][vargs[2].number().uintValue() - 1].representsNonMatrix();
+		}
+	}
+	return false;
+}
 int ElementFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	if(vargs[2].number().isPositive() && vargs[0].isMatrix()) {
-		size_t row = (size_t) vargs[1].number().uintValue();
-		size_t col = (size_t) vargs[2].number().uintValue();
-		bool b = true;
-		if(col > vargs[0].columns()) {
-			CALCULATOR->error(true, _("Column %s does not exist in matrix."), format_and_print(vargs[2]).c_str(), NULL);
-			b = false;
-		}
-		if(row > vargs[0].rows()) {
-			CALCULATOR->error(true, _("Row %s does not exist in matrix."), format_and_print(vargs[1]).c_str(), NULL);
-			b = false;
-		}
-		if(b) {
-			const MathStructure *em = vargs[0].getElement(row, col);
-			if(em) mstruct = *em;
-			else b = false;
-		}
-		return b;
-	} else {
-		if(vargs[2].number().isGreaterThan(1)) {
-			CALCULATOR->error(false, _("Argument 3, %s, is ignored for vectors."), getArgumentDefinition(3)->name().c_str(), NULL);
-		}
-		size_t row = (size_t) vargs[1].number().uintValue();
-		if(row > vargs[0].countChildren()) {
-			CALCULATOR->error(true, _("Element %s does not exist in vector."), format_and_print(vargs[1]).c_str(), NULL);
+	size_t row = (size_t) vargs[1].number().uintValue();
+	size_t col = (size_t) vargs[2].number().uintValue();
+	if(col == 0) {
+		if(row > vargs[0].size()) {
+			CALCULATOR->error(true, _("Element %s does not exist in vector."), format_and_print(vargs[0]).c_str(), NULL);
 			return 0;
 		}
-		mstruct = *vargs[0].getChild(row);
+		mstruct = vargs[0][row - 1];
 		return 1;
 	}
+	if(col > vargs[0].columns()) {
+		CALCULATOR->error(true, _("Column %s does not exist in matrix."), format_and_print(vargs[1]).c_str(), NULL);
+		return 0;
+	}
+	if(row > vargs[0].rows()) {
+		CALCULATOR->error(true, _("Row %s does not exist in matrix."), format_and_print(vargs[0]).c_str(), NULL);
+		return 0;
+	}
+	const MathStructure *em = vargs[0].getElement(row, col);
+	if(em) mstruct = *em;
+	else return 0;
+	return 1;
 }
 DimensionFunction::DimensionFunction() : MathFunction("dimension", 1) {
-	setArgumentDefinition(1, new VectorArgument(""));
+	setArgumentDefinition(1, new VectorArgument("", false));
 }
-int DimensionFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	mstruct = (int) vargs[0].countChildren();
+int DimensionFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
+	if(vargs[0].isVector()) {
+		if(vargs[0].isMatrix() && vargs[0].columns() == 1 && vargs[0].rows() > 1) {
+			mstruct.set((long int) vargs[0].rows(), 1L, 0L);
+			return 1;
+		} else if(vargs[0].representsNonMatrix()) {
+			mstruct.set((long int) vargs[0].countChildren(), 1L, 0L);
+			return 1;
+		}
+	}
+	mstruct = vargs[0];
+	mstruct.eval(eo);
+	if(!mstruct.isVector()) {
+		if(mstruct.representsScalar()) {
+			mstruct = m_one;
+			return 1;
+		} else if(eo.approximation == APPROXIMATION_EXACT || eo.approximation == APPROXIMATION_EXACT_VARIABLES) {
+			EvaluationOptions eo2 = eo;
+			eo2.approximation = APPROXIMATION_APPROXIMATE;
+			MathStructure m2(vargs[0]);
+			CALCULATOR->beginTemporaryStopMessages();
+			m2.eval(eo2);
+			if(CALCULATOR->endTemporaryStopMessages()) return -1;
+			if(m2.isVector()) {
+				if(mstruct.isMatrix() && mstruct.columns() == 1 && mstruct.rows() > 1) mstruct.set((long int) m2.rows(), 1L, 0L);
+				else mstruct.set((long int) m2.countChildren(), 1L, 0L);
+				return 1;
+			} else if(m2.representsScalar()) {
+				mstruct = m_one;
+				return 1;
+			}
+		}
+		return -1;
+	}
+	if(mstruct.isMatrix() && mstruct.columns() == 1 && mstruct.rows() > 1) mstruct.set((long int) mstruct.rows(), 1L, 0L);
+	else mstruct.set((long int) mstruct.countChildren(), 1L, 0L);
 	return 1;
 }
 ComponentFunction::ComponentFunction() : MathFunction("component", 2) {
 	setArgumentDefinition(1, new IntegerArgument("", ARGUMENT_MIN_MAX_POSITIVE, true, true, INTEGER_TYPE_SIZE));
 	setArgumentDefinition(2, new VectorArgument(""));
 }
+bool ComponentFunction::representsScalar(const MathStructure &vargs) const {
+	if(vargs.size() >= 2 && vargs[0].isVector() && vargs[1].isInteger() && vargs[1].number().isPositive() && vargs[1].number() <= vargs[0].size()) {
+		return vargs[0][vargs[1].number().uintValue() - 1].representsScalar();
+	}
+	return false;
+}
+bool ComponentFunction::representsNonMatrix(const MathStructure &vargs) const {
+	if(vargs.size() >= 2 && vargs[0].isVector() && vargs[1].isInteger() && vargs[1].number().isPositive() && vargs[1].number() <= vargs[0].size()) {
+		return vargs[0][vargs[1].number().uintValue() - 1].representsNonMatrix();
+	}
+	return false;
+}
 int ComponentFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
 	size_t i = (size_t) vargs[0].number().uintValue();
 	if(i > vargs[1].countChildren()) {
@@ -279,7 +419,10 @@ int TransposeFunction::calculate(MathStructure &mstruct, const MathStructure &va
 }
 IdentityMatrixFunction::IdentityMatrixFunction() : MathFunction("identity", 1) {
 	ArgumentSet *arg = new ArgumentSet();
-	arg->addArgument(new IntegerArgument("", ARGUMENT_MIN_MAX_POSITIVE, true, true, INTEGER_TYPE_SIZE));
+	IntegerArgument *iarg = new IntegerArgument("", ARGUMENT_MIN_MAX_POSITIVE, true, true, INTEGER_TYPE_SIZE);
+	Number nr(1, 1, 7);
+	iarg->setMax(&nr);
+	arg->addArgument(iarg);
 	MatrixArgument *marg = new MatrixArgument();
 	marg->setSquareDemanded(true);
 	arg->addArgument(marg);
@@ -315,7 +458,9 @@ int PermanentFunction::calculate(MathStructure &mstruct, const MathStructure &va
 	return !mstruct.isUndefined();
 }
 CofactorFunction::CofactorFunction() : MathFunction("cofactor", 3) {
-	setArgumentDefinition(1, new MatrixArgument());
+	MatrixArgument *marg = new MatrixArgument();
+	marg->setSquareDemanded(true);
+	setArgumentDefinition(1, marg);
 	setArgumentDefinition(2, new IntegerArgument("", ARGUMENT_MIN_MAX_POSITIVE, true, true, INTEGER_TYPE_SIZE));
 	setArgumentDefinition(3, new IntegerArgument("", ARGUMENT_MIN_MAX_POSITIVE, true, true, INTEGER_TYPE_SIZE));
 }
@@ -433,38 +578,42 @@ int NormFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 	}
 	return -1;
 }
-HadamardFunction::HadamardFunction() : MathFunction("hadamard", 1, -1) {
-	setArgumentDefinition(1, new VectorArgument());
-	setArgumentDefinition(2, new VectorArgument());
-}
-int HadamardFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions&) {
-	bool b_matrix = vargs[0].isMatrix();
-	for(size_t i3 = 1; i3 < vargs.size(); i3++) {
-		if(b_matrix) {
-			if(!vargs[i3].isMatrix() || vargs[i3].columns() != vargs[0].columns() || vargs[i3].rows() != vargs[0].rows()) {
-				CALCULATOR->error(true, _("%s() requires that all matrices/vectors have the same dimensions."), preferredDisplayName().name.c_str(), NULL);
-				return 0;
-			}
-		} else if(vargs[i3].size() != vargs[0].size()) {
-			CALCULATOR->error(true, _("%s() requires that all matrices/vectors have the same dimensions."), preferredDisplayName().name.c_str(), NULL);
-			return 0;
-		}
-	}
-	mstruct = vargs[0];
-	for(size_t i = 0; i < mstruct.size(); i++) {
-		if(b_matrix) {
-			for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
-				for(size_t i3 = 1; i3 < vargs.size(); i3++) mstruct[i][i2] *= vargs[i3][i][i2];
-			}
-		} else {
-			for(size_t i3 = 1; i3 < vargs.size(); i3++) mstruct[i] *= vargs[i3][i];
-		}
-	}
-	return 1;
-}
 bool matrix_to_rref(MathStructure &m, const EvaluationOptions &eo) {
 	size_t rows = m.rows();
 	size_t cols = m.columns();
+	if(cols > 0 && m[0][0].containsType(STRUCT_UNIT, false, false, false)) {
+		m[0][0].factorizeUnits();
+		MathStructure munit;
+		if(is_unit_multiexp(m[0][0])) {
+			munit = m[0][0];
+		} else if(m[0][0].isMultiplication()) {
+			munit = m[0][0];
+			for(size_t i = 0; i < munit.size();) {
+				if(!munit[i].containsType(STRUCT_UNIT, false, false, false)) {
+					munit.delChild(i + 1);
+				} else if(is_unit_multiexp(munit[i])) {
+					i++;
+				} else {
+					return false;
+				}
+			}
+			if(munit.size() == 1) munit.setToChild(1);
+			else if(munit.size() == 0) return false;
+		} else {
+			return false;
+		}
+		if(munit.isUnit_exp()) {
+			if(munit.isUnit() && munit.unit()->hasNonlinearRelationToBase()) return false;
+			else if(munit.isPower() && munit[0].unit()->hasNonlinearRelationToBase()) return false;
+		} else if(munit.isMultiplication()) {
+			for(size_t i = 0; i < munit.size(); i++) {
+				if(munit[i].isUnit() && munit[i].unit()->hasNonlinearRelationToBase()) return false;
+				else if(munit[i].isPower() && munit[i][0].unit()->hasNonlinearRelationToBase()) return false;
+			}
+		}
+		m.calculateDivide(munit, eo);
+		if(m.containsType(STRUCT_UNIT, false, false, false)) return false;
+	}
 	size_t cur_row = 0;
 	for(size_t c = 0; c < cols; ) {
 		bool b = false;
@@ -478,7 +627,7 @@ bool matrix_to_rref(MathStructure &m, const EvaluationOptions &eo) {
 				}
 				for(r = 0; r < rows; r++) {
 					if(r != cur_row) {
-						if(m[r][c].representsNonZero()) {
+						if(m[r][c].representsNonZero(true)) {
 							MathStructure mmul(m[r][c]);
 							mmul.calculateDivide(m[cur_row][c], eo);
 							mmul.calculateNegate(eo);
@@ -491,7 +640,7 @@ bool matrix_to_rref(MathStructure &m, const EvaluationOptions &eo) {
 									m[r][c2].calculateAdd(madd, eo);
 								}
 							}
-						} else if(!m[r][c].isZero()) {
+						} else if(!m[r][c].representsZero(true)) {
 							return false;
 						}
 					}
@@ -679,6 +828,7 @@ int EntrywiseMultiplicationFunction::calculate(MathStructure &mstruct, const Mat
 					if(mstruct[0].size() == m2[0].size()) {
 						for(size_t i = 0; i < mstruct.size(); i++) {
 							for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+								if(CALCULATOR->aborted()) return 0;
 								mstruct[i][i2] *= m2[i][i2];
 							}
 						}
@@ -686,9 +836,11 @@ int EntrywiseMultiplicationFunction::calculate(MathStructure &mstruct, const Mat
 					} else if(mstruct[0].size() == 1) {
 						for(size_t i = 0; i < m2.size(); i++) {
 							for(size_t i2 = 1; i2 < m2[i].size(); i2++) {
+								if(CALCULATOR->aborted()) return 0;
 								mstruct[i].addChild(mstruct[i][0]);
 							}
 							for(size_t i2 = 0; i2 < m2[i].size(); i2++) {
+								if(CALCULATOR->aborted()) return 0;
 								mstruct[i][i2] *= m2[i][i2];
 							}
 						}
@@ -696,6 +848,7 @@ int EntrywiseMultiplicationFunction::calculate(MathStructure &mstruct, const Mat
 					} else if(m2[0].size() == 1) {
 						for(size_t i = 0; i < mstruct.size(); i++) {
 							for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+								if(CALCULATOR->aborted()) return 0;
 								mstruct[i][i2] *= m2[i][0];
 							}
 						}
@@ -705,9 +858,11 @@ int EntrywiseMultiplicationFunction::calculate(MathStructure &mstruct, const Mat
 			} else if(mstruct.columns() == 1) {
 				for(size_t i = 0; i < mstruct.size(); i++) {
 					for(size_t i2 = 1; i2 < m2.size(); i2++) {
+						if(CALCULATOR->aborted()) return 0;
 						mstruct[i].addChild(mstruct[i][0]);
 					}
 					for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+						if(CALCULATOR->aborted()) return 0;
 						mstruct[i][i2] *= m2[i2];
 					}
 				}
@@ -717,16 +872,19 @@ int EntrywiseMultiplicationFunction::calculate(MathStructure &mstruct, const Mat
 			if(m2.isMatrix() && m2.columns() == 1) {
 				mstruct.transform(STRUCT_VECTOR);
 				for(size_t i = 1; i < m2.size(); i++) {
+					if(CALCULATOR->aborted()) return 0;
 					mstruct.addChild(mstruct[0]);
 				}
 				for(size_t i = 0; i < mstruct.size(); i++) {
 					for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+						if(CALCULATOR->aborted()) return 0;
 						mstruct[i][i2] *= m2[i][0];
 					}
 				}
 				return 1;
 			} else if(mstruct.size() == m2.size()) {
 				for(size_t i = 0; i < mstruct.size(); i++) {
+					if(CALCULATOR->aborted()) return 0;
 					mstruct[i] *= m2[i];
 				}
 				return 1;
@@ -763,10 +921,13 @@ int EntrywiseDivisionFunction::calculate(MathStructure &mstruct, const MathStruc
 			mstruct.clearVector();
 			if(m2.isMatrix()) {
 				mstruct.resizeVector(m2.size(), m_zero);
+				if(mstruct.size() < m2.size()) return 0;
 				for(size_t i = 0; i < m2.size(); i++) {
 					mstruct[i].clearVector();
 					mstruct[i].resizeVector(m2[i].size(), m);
+					if(mstruct[i].size() < m2[i].size()) return 0;
 					for(size_t i2 = 0; i2 < m2[i].size(); i2++) {
+						if(CALCULATOR->aborted()) return 0;
 						mstruct[i][i2] = m;
 						mstruct[i][i2] /= m2[i][i2];
 					}
@@ -774,7 +935,9 @@ int EntrywiseDivisionFunction::calculate(MathStructure &mstruct, const MathStruc
 				return 1;
 			} else {
 				mstruct.resizeVector(m2.size(), m);
+				if(mstruct.size() < m2.size()) return 0;
 				for(size_t i = 0; i < m2.size(); i++) {
+					if(CALCULATOR->aborted()) return 0;
 					mstruct[i] /= m2[i];
 				}
 				return 1;
@@ -787,6 +950,7 @@ int EntrywiseDivisionFunction::calculate(MathStructure &mstruct, const MathStruc
 						if(mstruct[0].size() == m2[0].size()) {
 							for(size_t i = 0; i < mstruct.size(); i++) {
 								for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+									if(CALCULATOR->aborted()) return 0;
 									mstruct[i][i2] /= m2[i][i2];
 								}
 							}
@@ -794,9 +958,11 @@ int EntrywiseDivisionFunction::calculate(MathStructure &mstruct, const MathStruc
 						} else if(mstruct[0].size() == 1) {
 							for(size_t i = 0; i < m2.size(); i++) {
 								for(size_t i2 = 1; i2 < m2[i].size(); i2++) {
+									if(CALCULATOR->aborted()) return 0;
 									mstruct[i].addChild(mstruct[i][0]);
 								}
 								for(size_t i2 = 0; i2 < m2[i].size(); i2++) {
+									if(CALCULATOR->aborted()) return 0;
 									mstruct[i][i2] /= m2[i][i2];
 								}
 							}
@@ -804,6 +970,7 @@ int EntrywiseDivisionFunction::calculate(MathStructure &mstruct, const MathStruc
 						} else if(m2[0].size() == 1) {
 							for(size_t i = 0; i < mstruct.size(); i++) {
 								for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+									if(CALCULATOR->aborted()) return 0;
 									mstruct[i][i2] /= m2[i][0];
 								}
 							}
@@ -813,9 +980,11 @@ int EntrywiseDivisionFunction::calculate(MathStructure &mstruct, const MathStruc
 				} else if(mstruct.columns() == 1) {
 					for(size_t i = 0; i < mstruct.size(); i++) {
 						for(size_t i2 = 1; i2 < m2.size(); i2++) {
+							if(CALCULATOR->aborted()) return 0;
 							mstruct[i].addChild(mstruct[i][0]);
 						}
 						for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+							if(CALCULATOR->aborted()) return 0;
 							mstruct[i][i2] /= m2[i2];
 						}
 					}
@@ -825,16 +994,19 @@ int EntrywiseDivisionFunction::calculate(MathStructure &mstruct, const MathStruc
 				if(m2.isMatrix() && m2.columns() == 1) {
 					mstruct.transform(STRUCT_VECTOR);
 					for(size_t i = 1; i < m2.size(); i++) {
+						if(CALCULATOR->aborted()) return 0;
 						mstruct.addChild(mstruct[0]);
 					}
 					for(size_t i = 0; i < mstruct.size(); i++) {
 						for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+							if(CALCULATOR->aborted()) return 0;
 							mstruct[i][i2] /= m2[i][0];
 						}
 					}
 					return 1;
 				} else if(mstruct.size() == m2.size()) {
 					for(size_t i = 0; i < mstruct.size(); i++) {
+						if(CALCULATOR->aborted()) return 0;
 						mstruct[i] /= m2[i];
 					}
 					return 1;
@@ -868,12 +1040,14 @@ int EntrywisePowerFunction::calculate(MathStructure &mstruct, const MathStructur
 			if(mstruct.isMatrix()) {
 				for(size_t i = 0; i < mstruct.size(); i++) {
 					for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+						if(CALCULATOR->aborted()) return 0;
 						mstruct[i][i2] ^= m2;
 					}
 				}
 				return 1;
 			} else {
 				for(size_t i = 0; i < mstruct.size(); i++) {
+					if(CALCULATOR->aborted()) return 0;
 					mstruct[i] ^= m2;
 				}
 				return 1;
@@ -886,10 +1060,13 @@ int EntrywisePowerFunction::calculate(MathStructure &mstruct, const MathStructur
 			mstruct.clearVector();
 			if(m2.isMatrix()) {
 				mstruct.resizeVector(m2.size(), m_zero);
+				if(mstruct.size() < m2.size()) return 0;
 				for(size_t i = 0; i < m2.size(); i++) {
 					mstruct[i].clearVector();
 					mstruct[i].resizeVector(m2[i].size(), m);
+					if(mstruct[i].size() < m2[i].size()) return 0;
 					for(size_t i2 = 0; i2 < m2[i].size(); i2++) {
+						if(CALCULATOR->aborted()) return 0;
 						mstruct[i][i2] = m;
 						mstruct[i][i2] ^= m2[i][i2];
 					}
@@ -897,7 +1074,9 @@ int EntrywisePowerFunction::calculate(MathStructure &mstruct, const MathStructur
 				return 1;
 			} else {
 				mstruct.resizeVector(m2.size(), m);
+				if(mstruct.size() < m2.size()) return 0;
 				for(size_t i = 0; i < m2.size(); i++) {
+					if(CALCULATOR->aborted()) return 0;
 					mstruct[i] ^= m2[i];
 				}
 				return 1;
@@ -909,6 +1088,7 @@ int EntrywisePowerFunction::calculate(MathStructure &mstruct, const MathStructur
 						if(mstruct[0].size() == m2[0].size()) {
 							for(size_t i = 0; i < mstruct.size(); i++) {
 								for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+									if(CALCULATOR->aborted()) return 0;
 									mstruct[i][i2] ^= m2[i][i2];
 								}
 							}
@@ -916,9 +1096,11 @@ int EntrywisePowerFunction::calculate(MathStructure &mstruct, const MathStructur
 						} else if(mstruct[0].size() == 1) {
 							for(size_t i = 0; i < m2.size(); i++) {
 								for(size_t i2 = 1; i2 < m2[i].size(); i2++) {
+									if(CALCULATOR->aborted()) return 0;
 									mstruct[i].addChild(mstruct[i][0]);
 								}
 								for(size_t i2 = 0; i2 < m2[i].size(); i2++) {
+									if(CALCULATOR->aborted()) return 0;
 									mstruct[i][i2] ^= m2[i][i2];
 								}
 							}
@@ -926,6 +1108,7 @@ int EntrywisePowerFunction::calculate(MathStructure &mstruct, const MathStructur
 						} else if(m2[0].size() == 1) {
 							for(size_t i = 0; i < mstruct.size(); i++) {
 								for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+									if(CALCULATOR->aborted()) return 0;
 									mstruct[i][i2] ^= m2[i][0];
 								}
 							}
@@ -935,9 +1118,11 @@ int EntrywisePowerFunction::calculate(MathStructure &mstruct, const MathStructur
 				} else if(mstruct.columns() == 1) {
 					for(size_t i = 0; i < mstruct.size(); i++) {
 						for(size_t i2 = 1; i2 < m2.size(); i2++) {
+							if(CALCULATOR->aborted()) return 0;
 							mstruct[i].addChild(mstruct[i][0]);
 						}
 						for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+							if(CALCULATOR->aborted()) return 0;
 							mstruct[i][i2] ^= m2[i2];
 						}
 					}
@@ -947,16 +1132,19 @@ int EntrywisePowerFunction::calculate(MathStructure &mstruct, const MathStructur
 				if(m2.isMatrix() && m2.columns() == 1) {
 					mstruct.transform(STRUCT_VECTOR);
 					for(size_t i = 1; i < m2.size(); i++) {
+						if(CALCULATOR->aborted()) return 0;
 						mstruct.addChild(mstruct[0]);
 					}
 					for(size_t i = 0; i < mstruct.size(); i++) {
 						for(size_t i2 = 0; i2 < mstruct[i].size(); i2++) {
+							if(CALCULATOR->aborted()) return 0;
 							mstruct[i][i2] ^= m2[i][0];
 						}
 					}
 					return 1;
 				} else if(mstruct.size() == m2.size()) {
 					for(size_t i = 0; i < mstruct.size(); i++) {
+						if(CALCULATOR->aborted()) return 0;
 						mstruct[i] ^= m2[i];
 					}
 					return 1;
@@ -1034,6 +1222,7 @@ bool process_matrix_replace(MathStructure &mprocess, const MathStructure &mstruc
 	}
 	bool b = false;
 	for(size_t i = 0; i < mprocess.size(); i++) {
+		if(CALCULATOR->aborted()) break;
 		if(process_matrix_replace(mprocess[i], mstruct, vargs, rindex, cindex)) {
 			mprocess.childUpdated(i + 1);
 			b = true;
@@ -1058,38 +1247,53 @@ int ProcessMatrixFunction::calculate(MathStructure &mstruct, const MathStructure
 	MathStructure mprocess;
 	for(size_t rindex = 0; rindex < mstruct.size(); rindex++) {
 		for(size_t cindex = 0; cindex < mstruct[rindex].size(); cindex++) {
-			if(CALCULATOR->aborted()) return 0;
 			mprocess = vargs[0];
 			process_matrix_replace(mprocess, mstruct, vargs, rindex, cindex);
+			if(CALCULATOR->aborted()) return 0;
 			mstruct[rindex][cindex] = mprocess;
 		}
 	}
 	return 1;
 
 }
-GenerateVectorFunction::GenerateVectorFunction() : MathFunction("genvector", 4, 6) {
+GenerateVectorFunction::GenerateVectorFunction() : MathFunction("genvector", 3, 6) {
+	setDefaultValue(4, "1");
 	setArgumentDefinition(5, new SymbolicArgument());
 	setDefaultValue(5, "undefined");
-	setArgumentDefinition(6, new BooleanArgument());
-	setDefaultValue(6, "0");
+	IntegerArgument *iarg = new IntegerArgument("");
+	iarg->setMax(&nr_one);
+	iarg->setMin(&nr_minus_one);
+	setArgumentDefinition(6, iarg);
+	setDefaultValue(6, "-1");
 }
 int GenerateVectorFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
 	if(CALCULATOR->aborted()) return 0;
-	if(vargs[5].number().getBoolean()) {
+	bool b_step = vargs[5].number().isPositive();
+	MathStructure msteps(vargs[3]);
+	if(!b_step) {
+		CALCULATOR->beginTemporaryStopMessages();
+		msteps.eval(eo);
+		if(vargs[5].number().isNegative()) {
+			b_step = !msteps.isInteger() || msteps.number().isNegative() || msteps.number().isOne();
+		}
+		CALCULATOR->endTemporaryStopMessages(!b_step);
+	}
+	if(b_step) {
 		mstruct = vargs[0].generateVector(vargs[4], vargs[1], vargs[2], vargs[3], NULL, eo);
 	} else {
 		bool overflow = false;
-		int steps = vargs[3].number().intValue(&overflow);
-		if(!vargs[3].isNumber() || overflow || steps < 1) {
+		int steps = msteps.number().intValue(&overflow);
+		if(!msteps.isNumber() || overflow || steps < 1) {
 			CALCULATOR->error(true, _("The number of requested elements in generate vector function must be a positive integer."), NULL);
 			return 0;
 		}
 		mstruct = vargs[0].generateVector(vargs[4], vargs[1], vargs[2], steps, NULL, eo);
 	}
-	if(CALCULATOR->aborted()) return 0;
+	if(CALCULATOR->aborted() || mstruct.size() == 0) return 0;
 	return 1;
 }
 SelectFunction::SelectFunction() : MathFunction("select", 2, 4) {
+	setArgumentDefinition(1, new VectorArgument());
 	setArgumentDefinition(3, new SymbolicArgument());
 	setDefaultValue(3, "undefined");
 	setArgumentDefinition(4, new BooleanArgument());
@@ -1098,31 +1302,14 @@ SelectFunction::SelectFunction() : MathFunction("select", 2, 4) {
 int SelectFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
 	MathStructure mtest;
 	mstruct = vargs[0];
-	mstruct.eval(eo);
-	if(!mstruct.isVector()) {
-		mtest = vargs[1];
-		mtest.replace(vargs[2], mstruct);
-		mtest.eval(eo);
-		if(!mtest.isNumber() || mtest.number().getBoolean() < 0) {
-			CALCULATOR->error(true, _("Comparison failed."), NULL);
-			return -1;
-		}
-		if(mtest.number().getBoolean() == 0) {
-			if(vargs[3].number().getBoolean() > 0) {
-				CALCULATOR->error(true, _("No matching item found."), NULL);
-				return -1;
-			}
-			mstruct.clearVector();
-		}
-		return 1;
-	}
 	for(size_t i = 0; i < mstruct.size();) {
+		if(CALCULATOR->aborted()) return 0;
 		mtest = vargs[1];
 		mtest.replace(vargs[2], mstruct[i]);
 		mtest.eval(eo);
 		if(!mtest.isNumber() || mtest.number().getBoolean() < 0) {
 			CALCULATOR->error(true, _("Comparison failed."), NULL);
-			return -1;
+			return 0;
 		}
 		if(mtest.number().getBoolean() == 0) {
 			if(vargs[3].number().getBoolean() == 0) {
@@ -1139,7 +1326,7 @@ int SelectFunction::calculate(MathStructure &mstruct, const MathStructure &vargs
 	}
 	if(vargs[3].number().getBoolean() > 0) {
 		CALCULATOR->error(true, _("No matching item found."), NULL);
-		return -1;
+		return 0;
 	}
 	return 1;
 }
@@ -1164,7 +1351,7 @@ int LoadFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 	return 1;
 }
 ExportFunction::ExportFunction() : MathFunction("export", 2, 3) {
-	setArgumentDefinition(1, new VectorArgument());
+	setArgumentDefinition(1, new MatrixArgument());
 	setArgumentDefinition(2, new FileArgument());
 	setArgumentDefinition(3, new TextArgument());
 	setDefaultValue(3, "\",\"");
